///////////////////////////////////////////////////////////////////////////////
// sym2name.cpp : 
// ----------------------------------------------------------------------------
// Modifiziert fr den Gebrauch der STL 
// dieses Programm steht unter der GPL V3.0
///////////////////////////////////////////////////////////////////////////////

#include <streambuf>
#include <iostream>
#include <fstream>
#include <string>

#include <stdio.h>
#include <stdlib.h>

using namespace std ;

#define MAXSYM 1000
#define vers "1.06"
#define csvvers 2

void help(void)
{
	cerr <<"Sym2Name Vers.:" << vers <<"\n\n";
	cerr <<"This Program is free software under GPL V.3\n";
	cerr <<"A copy of that license should be provided with this program\n";
	cerr <<"If not please get ane at <http://www.gnu.org/licenses/gpl.html>\n\n";
	cerr <<"usage: sym2name [infile] [-i:infile] [-o:outfile] [-c:configurationfile]\n";
	cerr <<"This program adds the name of a waypoint symbol to the waypoint name\n";
	cerr <<"You need to provide at least the path of the original file,\n";
	cerr <<"either with or without '-i:'\n";
	cerr <<"If you do not provide a outputfile a '_s2n' will be added to the infilename.\n";
	cerr <<"If you do not provide a configurationfile, sym2name.csv will be used\n";
	cerr <<"this way you can just drop the infile on the program\n";
	cerr <<"For more information see the readme.pdf provided\n";
	cerr <<"Or if you speek german check <http://wiki.openstreetmap.org/wiki/Sym2name>\n";
	cerr <<"example 1: sym2name example.gpx\n";
	cerr <<"     example.gpx will be transformed into example_s2n.gpx using sym2name.csv\n";
	cerr <<"example 2: sym2name -i:example.gpx -o:output.gpx -c:my.csv\n";
	cerr <<"     example.gpx will be transformed into output.gpx using my.csv\n\n";
}

void error( const char *s, const char *s2="", const char *s3="")
{
	cerr << s << ' ' << s2 << ' ' <<s3 << "\nPress Return to exit\n";
	getchar();
	exit(1);
}

int main(int argc, char* argv[])
{
	string oFileName="",iFileName="",lFileName="",sFileName="sym2name.csv";
	int err=0;
	if(argc<2)
	{
		help();
		error("too few arguments");
	}
	else
	{
		int idx;
		sFileName=argv[0];
		idx=sFileName.find_last_of("\\");
		if(idx<string::npos)
		{
			sFileName.replace(idx+1,string::npos,"sym2name.csv");
			lFileName=argv[0];
			lFileName.replace(lFileName.find_last_of("\\")+1,string::npos,"sym2name.log");
		}
		else
		{
			sFileName="sym2name.csv";
			lFileName="sym2name.log";
		}
		for(int i=1;i<argc;i++)
		{
			if(argv[i][0]=='-')
			{
				switch(argv[i][1])
				{
				case 'i':
				case 'I':
						iFileName=argv[i]+3;
						break;
				case 'o':
				case 'O':
						oFileName=argv[i]+3;
						break;
				case 'c':
				case 'C':
						sFileName=argv[i]+3;
						break;
				default:
					error("unknown parameter",argv[i]);
				}
			}
			else
				iFileName=argv[i];
		}
	}
	
	if(iFileName.empty())
	{
		help();
		error("No source file");
	}
	if(oFileName.empty())
	{
		int idx=iFileName.rfind('.');
		if(idx>0 && idx<string::npos)
		{
			oFileName.append ( iFileName, 0, idx );
			oFileName += "_s2n.gpx";
		}
		else
			oFileName=iFileName+"_s2n";
	}
	ofstream logSt;
	logSt.open( lFileName.c_str());
	logSt<<"Parameter: Input:  "<<iFileName<<"\n";
	logSt<<"           Output: "<<oFileName<<"\n";
	logSt<<"           Conf:   "<<sFileName<<"\n";

	if(!err)
	{
		int anzSym=0;
		string sym[MAXSYM],name[MAXSYM],nameAkt="",wpAkt="",wpNxt="",trenner=",;";

		ifstream iniSt;
		iniSt.exceptions( ifstream::failbit | ifstream::badbit );

		try
		{
			// ios::in ist default
			// textmode auch
			int version=0;
			bool first=true;

			iniSt.open( sFileName.c_str());
			iniSt.exceptions( ifstream::badbit);
	
			string zeile;

			// Datei zeilenweise auslesen
			while ( getline( iniSt, zeile ))
			{
				if(first && zeile.find("#Version")<2)
				{
					int idx=zeile.find_first_of("0123456789");
					if(idx>-1)
					{
						version=atoi(zeile.substr(idx).c_str());
						if(version>csvvers)

							error("the csv-file has a version higher than this program, please update");
						if(version>1)
						{
							int idx2=zeile.find_first_of(trenner);
							if(idx2<idx)
							{
								trenner="";
								trenner.append(zeile.substr(idx2,1));
							}
						}
						logSt<<"Version:"<<version<<"\n";
						logSt<<"Separator: "<<trenner <<"\n";
					}
					first=false;
				}
				else if(zeile[0]!=';' && zeile.find("\";")>0) //sonst Kommentarzeile
				{
					int start=zeile.find_first_not_of(" \t");	// Leerstellen und Tabs berspringen
					int komma=string::npos,ende;
					if(zeile[start]=='\"')
					{
						start++;				//Anfhrungszeichen berspringen
						start=zeile.find_first_not_of(" \t",start);	// Anzahl Leerstellen und Tabs am Anfang
						ende=zeile.find('\"',start);
						while(ende<string::npos && zeile[ende+1]=='\"') // "" entsprechen einem Anfhrungszeichen im String
							ende=zeile.find('\"',ende+2);
						if(ende<string::npos)
						{
							sym[anzSym]=zeile.substr(start,ende-start);
							start=0;
							while((start=sym[anzSym].find("\"\"",start))<string::npos)	// "" durch " ersetzen
								sym[anzSym].erase(start++,1);
							komma=zeile.find_first_of(trenner,ende);
						}
					}
					else 
					{
						komma=zeile.find_first_of(trenner);
						sym[anzSym]=zeile.substr(start, komma-start);
					}
					if(komma<string::npos)
					{
						start=komma+1;
						sym[anzSym].erase(sym[anzSym].find_last_not_of(" \t")+1,string::npos);	// Lscht Leerstellen und Tabs am Ende		
						start=zeile.find_first_not_of(" \t",start);	// Anzahl Leerstellen und Tabs 
						if(zeile[start]=='\"')
						{
							start++;				//Anfhrungszeichen berspringen
							ende=zeile.find('\"',start);
							while(ende<string::npos && zeile[ende+1]=='\"') // "" entsprechen einem Anfhrungszeichen im String
								ende=zeile.find('\"',ende+2);
							if(ende<string::npos)
							{
								name[anzSym]=zeile.substr(start,ende-start);
								start=0;
								while((start=name[anzSym].find("\"\"",start))<string::npos)	// "" durch " ersetzen
									name[anzSym].erase(start++,1);
								anzSym++;
							}
						}
						else
						{
							ende=zeile.find_first_of(trenner,start);
							name[anzSym++]=zeile.substr(start,ende-start);
						}
					}
				}
			}
			iniSt.close();
		}
		catch ( const ios::failure &failure )
		{

// to do : ios:failure nach dem Dateinamen durchsuchen
			error( "Can't read",sFileName.c_str(), failure.what());
		}
		logSt<<"Translation:\n";
		for(int i=0;i<anzSym;i++)
			logSt<<i<<" : "<<sym[i]<<" -> "<<name[i]<<"\n";


// gpx Datei kopieren mit nderungen


		ifstream fromSt;
		fromSt.exceptions( ifstream::failbit | ifstream::badbit );
		ofstream toSt;
		toSt.exceptions( ofstream::failbit | ofstream::badbit );
		try
		{
			int inWpt=0;
			fromSt.open( iFileName.c_str());
			toSt.open( oFileName.c_str());
			fromSt.exceptions( ifstream::badbit );
			string zeile;

			// Datei zeilenweise auslesen
			while (getline( fromSt, zeile ))
			{
   				int nxtPos,symPosA,symPosE,namePos,wpPosE;
				wpAkt+=zeile;
				wpAkt+="\n";
				nxtPos=wpAkt.find("<wpt",inWpt);	//neuer Wegpunkt
				if(nxtPos==0)	//dann braucht man die Zeile nicht aufzuteilen
				{
					inWpt=1;
					nxtPos=wpAkt.find("<wpt",inWpt);	//neuer Wegpunkt
				}
				if(nxtPos<string::npos)
				{
					wpNxt=wpAkt.substr(nxtPos);
					wpAkt=wpAkt.substr(0, nxtPos);
				}
				if(inWpt )	// Wegpunkt
				{
					symPosA=wpAkt.find("<sym>");
					symPosE=wpAkt.find("</sym>",symPosA+5);
					wpPosE=wpAkt.find("</wpt>",symPosE+5);
					if(symPosA<string::npos && symPosE<string::npos && wpPosE<string::npos)	//es gibt nur Arbeit wenn es ein Wegpunkt mit <sym> ist
					{
						symPosA=wpAkt.find_first_not_of(" \t",symPosA+5);	// Leerstellen und Tabs berspringen
						symPosE=wpAkt.find_last_not_of(" \t",symPosE);	// Leerstellen und Tabs berspringen
						string symAkt=wpAkt.substr(symPosA,symPosE-symPosA);
						for(int i=0;i<anzSym;i++)
						{
							if(!symAkt.compare(sym[i]))
							{
								symAkt=name[i];
								break;
							}
						}
						namePos=wpAkt.find("<name>");
						if(namePos<string::npos)					// entweder in bestehenden Namen einfgen
						{
							wpAkt.insert(namePos+6," / ");
							wpAkt.insert(namePos+6,symAkt);
						}
						else
						{								// oder neuen Namen anlegen
							wpAkt.insert(wpPosE,"</name>");
							wpAkt.insert(wpPosE,symAkt);
							wpAkt.insert(wpPosE,"\n<name>");
						}
					}
				}
				if(wpPosE<string::npos||!wpNxt.empty()||!inWpt)
				{
					toSt.write(wpAkt.c_str(),wpAkt.length());		// Wegpunkt oder uninteressantes schreiben
					if(wpNxt.empty())
					{
						wpAkt="";
						inWpt=0;
					}
					else
					{
						wpAkt=wpNxt;
						wpNxt="";
						inWpt=1;
					}
				}
			}
			toSt.write(wpAkt.c_str(),wpAkt.length());		// Rest schreiben
			toSt.flush();
			toSt.close();
			fromSt.close();
		}
		
		catch(const ios::failure &failure )
		{

// to do : ios:failure nach dem Dateinamen durchsuchen

			error("IO-Error with file",failure.what());

		}
	}
	return err;
}



